/*-----------------------------------------------------------------------------
	Paper Plane xUI  Windows Search Module  Copyright (c)TORO 2010,2025
-----------------------------------------------------------------------------*/
#define STRICT
#ifndef UNICODE
#define UNICODE
#endif
#if _MSC_VER >= 1900
#undef WINVER
#define WINVER 0x0600
#endif
#include <windows.h>
#include <winnls.h>
#include <atlbase.h>
#include <atldbcli.h>
#include <searchapi.h>
#include "torowin.h"
#include "ppcommon.h"

#define MODULEVERSIONSTR L"R2"
/*-----------------------------------------------------------------------------
	dll ̏^I
-----------------------------------------------------------------------------*/
BOOL WINAPI DllMain(HINSTANCE, DWORD, LPVOID)
{
	return TRUE;
}

WCHAR *stplimcpyW(WCHAR *deststr, const WCHAR *srcstr, WCHAR *destmax)
{
	WCHAR code;

	destmax--;
	while( deststr < destmax ){
		code = *srcstr++;
		if ( code == '\0' ) break;
		*deststr++ = code;
	}
	*deststr = '\0';
	return deststr;
}

// ISearchQueryHelper 擾
static HRESULT GetQueryHelper(ISearchQueryHelper **ppQueryHelper)
{
	ISearchManager *pSearchManager;
	HRESULT hr;

	*ppQueryHelper = NULL;
	hr = CoCreateInstance(__uuidof(CSearchManager), NULL, CLSCTX_LOCAL_SERVER, IID_PPV_ARGS(&pSearchManager));
	if (SUCCEEDED(hr)){
		ISearchCatalogManager *pSearchCatalogManager;
		hr = pSearchManager->GetCatalog(L"SystemIndex", &pSearchCatalogManager);
		if (SUCCEEDED(hr)){
			hr = pSearchCatalogManager->GetQueryHelper(ppQueryHelper);
			pSearchCatalogManager->Release();
		}
		pSearchManager->Release();
	}
	return hr;
}

// ISearchQueryHelper gČpSQL擾
static HRESULT GetSQLString(const WCHAR *keyword, PWSTR *ppszSQL, LONG maxresults)
{
	HRESULT hr;

	*ppszSQL = NULL;
	ISearchQueryHelper *pQueryHelper;
	hr = GetQueryHelper(&pQueryHelper);
	if (SUCCEEDED(hr)){
		pQueryHelper->put_QueryMaxResults(maxresults);
		pQueryHelper->put_QuerySelectColumns(L"System.ItemPathDisplay");
		hr = pQueryHelper->GenerateSQLFromUserQuery(keyword, ppszSQL);
		pQueryHelper->Release();
	}
	return hr;
}

static void GetResultString(WCHAR **dest, WCHAR *destmax, DBTYPE type, void *value)
{
	switch (type) {
		case DBTYPE_VARIANT: {
			VARIANT *pVar = (VARIANT *)value;

			if ( pVar->vt == VT_BSTR ){
				*dest = stplimcpyW(*dest, pVar->bstrVal, destmax);
				return;
			}
			break;
		}

		case DBTYPE_BSTR:
			*dest = stplimcpyW(*dest, (BSTR)value, destmax);
			return;

		case DBTYPE_WSTR:
			*dest = stplimcpyW(*dest, (PCWSTR)value, destmax);
			return;

//		default:
	}
	*dest += wsprintf(*dest, L"(Unhandled database type %d)", type);
}

static HRESULT DoWindowsSearch(PPXMSEARCHSTRUCT *pss, PPXAPPINFOW *ppxa)
{
	PWSTR SQLstring;
	WCHAR buf[0x1000], *dest;
	HRESULT hr;
	CDataSource cDataSource;

	hr = GetSQLString(pss->keyword, &SQLstring, pss->maxresults);
	if ( FAILED(hr) ) return hr;

	hr = cDataSource.OpenFromInitializationString(L"provider=Search.CollatorDSO.1;EXTENDED PROPERTIES=\"Application=Windows\"");
	if ( SUCCEEDED(hr) ){
		CSession cSession;

		if ( SUCCEEDED(cSession.Open(cDataSource)) ){
			CCommand<CDynamicAccessor, CRowset> cCommand;

			if ( SUCCEEDED(cCommand.Open(cSession, SQLstring)) ){
				for ( hr = cCommand.MoveFirst() ; hr == S_OK ; hr = cCommand.MoveNext()){
					dest = buf;
					for ( DBORDINAL i = 1 ; i <= cCommand.GetColumnCount() ; i++ ){
						DBSTATUS status;

						cCommand.GetStatus(i, &status);
						if ( (status == DBSTATUS_S_OK) || (status == DBSTATUS_S_TRUNCATED) ){
							DBTYPE type;

							cCommand.GetColumnType(i, &type);
							GetResultString(&dest, dest + TSIZEOFW(buf), type, cCommand.GetValue(i));
						}else{
							dest = stplimcpyW(dest, L"(Error reading result)", dest + TSIZEOFW(buf));
						}
					}
					*dest = '\0';
					ppxa->Function(ppxa, PPXCMDID_REPORTSEARCH, buf);
				}
				if ( hr == DB_S_ENDOFROWSET) hr = S_FALSE;  // no rows
				cCommand.Close();
			}
			cCommand.ReleaseCommand();
		}
	}
	CoTaskMemFree(SQLstring);
	return hr;
}
/*-----------------------------------------------------------------------------
	{
-----------------------------------------------------------------------------*/
#define RESULT_SEPARATER 0
extern "C" int PPXAPI ModuleEntry(PPXAPPINFOW *ppxa, DWORD cmdID, PPXMODULEPARAM pxs)
{
	if ( cmdID == PPXMEVENT_SEARCH ){
#if RESULT_SEPARATER
		WCHAR buf[1000];

		wsprintfW(buf, L"WinSearch Keyword = %s", pxs.search->keyword);
		ppxa->Function(ppxa, PPXCMDID_REPORTSEARCH, buf);
#endif
		if ( pxs.search->keyword[0] != '\0' ){
			HRESULT hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);

			if ( SUCCEEDED(hr) || (hr == RPC_E_CHANGED_MODE) ){
				DoWindowsSearch(pxs.search, ppxa);
				if ( SUCCEEDED(hr) ) CoUninitialize();
			}
		}
#if RESULT_SEPARATER
		ppxa->Function(ppxa, PPXCMDID_REPORTSEARCH, L"-end-");
#endif
		return PPXMRESULT_SKIP; // ̃W[
	}

	if ( cmdID == PPXM_INFORMATION ){
		if ( pxs.info->infotype == 0 ){
			pxs.info->typeflags = PPMTYPEFLAGS(PPXM_INFORMATION) |
					PPMTYPEFLAGS(PPXMEVENT_SEARCH);
			wcscpy(pxs.info->copyright, L"PPx Windows Search Module " MODULEVERSIONSTR L"  Copyright (c)TORO");
			return PPXMRESULT_DONE;
		}
	}
	return PPXMRESULT_SKIP;
}
